/* 
 * File:   20200410_K269B_MK204B_1825.c
 * Author: cnet_fmv
 *
 * Created on 2020/04/10, 14:28
 */
/***************************************************
* By CCSC compiler
*  File Name: 20200410_MK204B_TalkingTester.c
*  Description: check Volt & Amp with MCP3425 and play ATP3011
*  RA5 out to ATP3011 RESET
*  RA4 out to ATP3011 RXD
*  RA2 in to PLAY, active low
*  RA1 in to Volt/Amp, low means Amp
*  RA0 in to x1 / x10 Volt, 
low means x10
*  RC5 out to LED, active high
*  RC4 in to ATP3011 play satatus
*  RC1 out to MCP3421 SCL
*  RC0 in/out to MCP3421 SDA
*  RC2 AD input for repeat period setting potentiometer
*  COPYRIGHT 2020 MYCOMKITS.COM, owned by CNET LIMITED
***************************************************/
//
#include <16f1825.h>
#DEVICE ADC=8
#fuses	INTRC_IO, NOWDT, MCLR, NOLVP, NOPROTECT
#use	delay(CLOCK = 4000000)
#use	rs232(BAUD = 9600, XMIT = PIN_A4)
#use I2C(Master, SLOW, SDA = PIN_C1, SCL = PIN_C0, ADDRESS=0xA0)

// initial data on EEPROM
#ROM getenv("EEPROM_ADDRESS") = {0x7F, 0x08, 0xF5, 0xC3, 0x7F, 0, 0, 0}//
//
#define Amode 0x0F			//port A 
#define Cmode 0x17			//port C
#define LED PIN_C5			//
#define ATP_RESET PIN_A5			//
#byte OSCCON = 0x099
#byte WPUA = 0x20C
#byte OPTION_REG = 0x095

// variables
int low_buffer = 0;
int high_buffer = 0;
long adc_in = 0;	//16 bits
long preadc_in = 0;
float value = 0; //actual value that is converted
int mmode = 0;	// 1=amp, 2=voltx1, 3=voltx10 
int en_repeat = 0;	// 1 = enable auto repeat every 5 sec
float thres;
int repeat_time = 50;
int minusflag = 0;	// 1 menas minus value
int value_setting = 50;	// max 256
int pre_value = 50;
int fast_repeat = 0;	// 1 means repeating fast without unit
float volt10_coef = 1.07;	// x10 volt coefficient
float amp_coef = 1.00;	// amp coefficient
//float amp_coef = 1.04;	// amp coefficient
float dummy;
long int j;
int factory_v_data = 0;
int factory_a_data = 0;
float talked_value = 0;

// prototyping
#separate
void initializing();
#separate
void readadc(); 
#separate
void tellvalue();
#separate
void tell31();
#separate
void tell13();
#separate
void get_setting();
#separate
void init_coeff();
#separate
void WRITE_FLOAT_EEPROM();
#separate
void READ_FLOAT_EEPROM();	
#separate
void blink_led();

//write_float_from_eeprom;
#separate
void WRITE_FLOAT_EEPROM(int addr, float data) 
{
   int8 i;
   for (i = 0; i < 4; i++) 
	{
		write_eeprom(addr + i, *((int8*)&data + i));
		delay_ms(20);
	}  
}

//read_float_from_eeprom;
#separate
float READ_FLOAT_EEPROM(int addr) 
{   
	int8 i;
	float data=0;
	for (i = 0; i < 4; i++) 
	{      
		*((int8*)&data + i) = read_eeprom(addr + i);
		delay_ms(20);      
	}
	return(data);
}

// timer 1 sub function
// wait time count down the timer value at every 100msec.
#int_timer1
void isr1(void)	
{
	set_timer1(0x7FFF);             // setting timer 1. almost 100msec
	if(repeat_time > 0) repeat_time = repeat_time -1;
	else repeat_time = 0;
}	

// main function
void main()
{
	initializing();
	
	// main loop
	while(1)
	{
		//
		get_setting();	// get repeat value from potentio meter
		//check mode sw
		if(input(PIN_A1) == 0) 	// check mode sw, A1 shows amp
		{
			if(mmode != 1) 
			{
				//	
				delay_ms(1000);
				while(input(PIN_C4)==0) {}	//busy
				disable_interrupts(GLOBAL); 
				printf("den'ryuusokuteimo'-dodesu");
				putc(0x0D);
				//
				i2c_start();
				i2c_write(0xD0); //write mode
				//16bit, continuous conversion, PGA = 8V/V
				i2c_write(0x1B);	// mode 1, amp mode
				i2c_stop();
				//
				enable_interrupts(GLOBAL);
			}	
			mmode = 1;	// amp mode
			thres = 232;	// 8*0x7FFF/1000, 1% of full range, actual adc value
 		}	
		else if(input(PIN_A0) == 0)	// check mode sw, A0 shows x10 volt
		{
			if(mmode != 3) 
			{
				//	
				delay_ms(1000);
				while(input(PIN_C4)==0) {}	//busy
				disable_interrupts(GLOBAL);
				printf("den'a'tusokuteimo'-do/ nijyu'-bo'rutore'njidesu");
				putc(0x0D);
				//
				i2c_start();
				i2c_write(0xD0); //write mode, address 0
				//16bit, continuous conversion, PGA = 1V/V
				i2c_write(0x18);	// mode 3, volt x10 mode
				i2c_stop();
				enable_interrupts(GLOBAL);
			}
			mmode = 3;	// volt x10 mode
			thres = 327;	// 32764/100, 1% of full range, actual adc value
		}	
		else 
		{
			if(mmode != 2) 
			{
				//	
				delay_ms(1000);
				while(input(PIN_C4)==0) {}	//busy
				disable_interrupts(GLOBAL);
				printf("den'atusokuteimo'-do/ nibo'rutore'njidesu");
				putc(0x0D);
				//
				i2c_start();
				i2c_write(0xD0); //write mode
				//16bit, continuous conversion, PGA = 1V/V
				i2c_write(0x18);	// mode 2, volt x1 mode
				i2c_stop();
				enable_interrupts(GLOBAL);
			}
			mmode = 2;	// volt x1 mode
			thres = 327;	// 32764/100, 1% of full range, actual adc value
		}	
		//			
		// get value from MCP3421 and play it by one of 3 ways 
		//while(input(PIN_C4)==0) {}	//busy
		readadc();
		// play it, if over threshold
		if((adc_in > (preadc_in + thres)) || (adc_in < (preadc_in - thres)))
		{
			delay_ms(200);
            readadc();
            // play
			tellvalue();
			delay_ms(1000);
			readadc();
			tellvalue();
		}
		// play it, if repeat time = 0
		else if((repeat_time == 0) && (en_repeat == 1))	// repeat time will be decremented by int1
		{
			// play
			tellvalue();
			repeat_time = value_setting;	// 5sec = 50x100msec repeat cycle
		}	
		// play it, if pressing PLAY switch
		else if(input(PIN_A2) == 0)
		{
			// play
			tellvalue();
		}
	}	
//	
}

#separate
void initializing() 
{
	//OSCCON = 0xEA;	// set 4MHz

	set_tris_a(Amode);	//
	set_tris_c(Cmode);	//
	
    port_a_pullups(0x3F);
	
	// A/D converter initialize AN6, RC2
	setup_adc_ports(sAN6 | VSS_VDD);	// gnd to 5v
	setup_adc(ADC_CLOCK_DIV_32);	
	
	// initialize timer 1 for repeat play
	setup_timer_1(T1_INTERNAL|T1_DIV_BY_8);
	set_timer1(0x7FFF);              // setting almost 104.4msec
	enable_interrupts(INT_TIMER1);   // enable timer 1
	
	// reset ATP3011
	output_high(ATP_RESET);
    delay_ms(200);
	output_low(ATP_RESET);
	delay_ms(300);
	output_high(ATP_RESET);
	delay_ms(100);
    blink_led();
    
	//first, configure ADC
	//output_float(PIN_C0);
	//output_float(PIN_C1);
	//
	i2c_start();
	i2c_write(0xD0); //write mode
	//16bit, continuous conversion, PGA = 8V/V
	if(input(PIN_A1) == 0) i2c_write(0x1B);	// mode 1, amp mode
	//16bit, continuous conversion, PGA = 1V/V
	else i2c_write(0x18);
	//i2c_write(0x1C);	// for 18 bits
	i2c_stop();
	//
	delay_us(100);
	
	//
	printf("?");	//set baud rate
	delay_ms(100);
	//
	init_coeff();	// initialize volt10 and amp coefficient
	//
	printf("to-kinngu/te'suta-/kido'-shima'shita");
	putc(0x0D);
	//
	delay_ms(2000);
	//
	// LED off
	output_low(PIN_C5);
	//
	// get initial value for repeat time
	set_adc_channel(6);	//must wait 65u, AN6, RC2
	delay_us(100);
	pre_value = read_adc();
	repeat_time = 0;
	//
	enable_interrupts(GLOBAL);
	//

}		
//
#separate
void readadc()
{
	//
	disable_interrupts(GLOBAL);
	//  
	i2c_start();
	i2c_write(0xD1);//Read mode
	high_buffer = i2c_read(1);
	low_buffer = i2c_read(0);
	i2c_stop();
	//
	enable_interrupts(GLOBAL);
	//
	// adc_value 16 bit length
	adc_in = high_buffer;
	adc_in <<= 8;
	adc_in = adc_in + low_buffer;
	if((adc_in & 0x8000) == 0) minusflag = 0;
	else 
	{
		minusflag = 1;
		adc_in = 1-adc_in;	//  get complement
	}	
	//
	value = adc_in;	// value is float, adc_in is long int
	//
}	
//
#separate
void tellvalue() 
{
	//
	preadc_in = adc_in;	// keep talked value
	// LED on
	output_high(PIN_C5);
	//
	if(mmode == 1)
	{
		// amp mode
		value = value*2.559/0x7FFF;	//value is 0 to 26214
		//
		if(value > 2.04)
		{
			disable_interrupts(GLOBAL);
			printf("#K");
			putc(0x0D);
			delay_ms(1000);
			printf("chu'ui/ saida'itio/koeteima'su");
			putc(0x0D);
			enable_interrupts(GLOBAL);
		}
		else
		{	
			if(minusflag == 1) 
			{
				value = value * amp_coef; // adjust for internal impedance
			}	
			else value = value * amp_coef; // adjust for internal impedance
			// adjust for internal impedance
			if(value < 1.0)
			{
				value = value * 1000;
				if(minusflag == 1) printf("mainasu");
				tell31();
				disable_interrupts(GLOBAL);
				printf("miri'anpea");
				putc(0x0D);
				enable_interrupts(GLOBAL);
			}	
			else
			{
				if(minusflag == 1) printf("mainasu");
				// over or equal 1.0
				tell13();
				disable_interrupts(GLOBAL);
				printf("anpea");
				putc(0x0D);
				enable_interrupts(GLOBAL);
			}
		}
	}		
	else if(mmode == 2)
	{
		// volt x1 mode
		value = value*2.048/0x7FFF;	//value is 0 to 32767
		//
		if(value > 2.04)
		{
			disable_interrupts(GLOBAL);
			printf("#K");
			putc(0x0D);
			delay_ms(1000);
			printf("chu'ui/ saida'itio/koeteima'su");
			putc(0x0D);
			enable_interrupts(GLOBAL);
		}
		else
		{	
			if(value < 1)
			{
				value = value * 1000;
				if(minusflag == 1) printf("mainasu");
				tell31();
				disable_interrupts(GLOBAL);
				printf("miri'boruto");
				putc(0x0D);
				enable_interrupts(GLOBAL);
			}	
			else
			{
				if(minusflag == 1) printf("mainasu");
				// over or equal 1.0
				tell13();
				disable_interrupts(GLOBAL);
				printf("bo'ruto");
				putc(0x0D);
				enable_interrupts(GLOBAL);
			}
		}
	}	
	else if(mmode == 3)
	{
		// volt x10 mode
		float tempo;
		value = value*2.048/0x7FFF;	//value is 0 to 32767
		//
		if(value > 2.04)
		{
			disable_interrupts(GLOBAL);
			printf("#K");
			putc(0x0D);
			delay_ms(1000);
			printf("chu'ui/ saida'itio/koeteima'su");
			putc(0x0D);
			enable_interrupts(GLOBAL);
		}
		else
		{	
			if(minusflag == 1) 
			{
				disable_interrupts(GLOBAL);
				printf("mainasu");
				enable_interrupts(GLOBAL);
				// adjust for internal impedance
				value = 10 * value * volt10_coef; // adjust for internal impedance
			}	
			else value = 10 * value * volt10_coef; // adjust for internal impedance
			//
			if(value < 1)
			{
				value = value * 1000;
				tell31();
				disable_interrupts(GLOBAL);
				printf("miri'boruto");
				putc(0x0D);
				enable_interrupts(GLOBAL);
			}	
			else
			{
				// over or equal 1.0
				disable_interrupts(GLOBAL);
				putc(0x3C);
				printf("NUMK VAL=");
				printf("%2.2f",value);
				putc(0x3E);
				printf("bo'ruto");
				putc(0x0D);
				enable_interrupts(GLOBAL);
			}
		}
	}
	//	
	delay_ms(2000);
	while(input(PIN_C4)==0) {}	//busy
	// LED off
	output_low(PIN_C5);
}
//	
//
#separate
void tell31() 
{
	disable_interrupts(GLOBAL);
	putc(0x3C);
	printf("NUMK VAL=");
	printf("%3.1f",value);
	putc(0x3E);
	enable_interrupts(GLOBAL);
}	
//
#separate
void tell13() 
{
	disable_interrupts(GLOBAL);
	putc(0x3C);
	printf("NUMK VAL=");
	printf("%1.3f",value);
	putc(0x3E);
	enable_interrupts(GLOBAL);
	//
}	

// get repeat time value from potentiometer
#separate
void get_setting()
{
	set_adc_channel(6);	//must wait 65u, AN6, RC2
	delay_us(100);
	value_setting=read_adc();	// 0 to 1024, 0V to 5V
	if((value_setting > pre_value + 5) || (value_setting < pre_value - 5)) repeat_time = 0;
	pre_value = value_setting;	// 0 to 1024, 0V to 5V
	if(pre_value < 5) pre_value = 5;
	if(pre_value > 250) pre_value = 250;
	if(value_setting < 245)
	{
		en_repeat = 1;
		if(value_setting < 15) 
		{
			fast_repeat = 1;
			value_setting = 0;
		}	
		else fast_repeat = 0;
	}
	else en_repeat = 0;	
	//en_repeat = 0;
}
// load stored coefficient data for x10 voltage and amp mode
#separate
void init_coeff() 
{
	if(input(PIN_A2) == 1)	// if pressing PLAY sw, then calibrate
	{
		volt10_coef = READ_FLOAT_EEPROM(0);	// load stored data
		amp_coef = READ_FLOAT_EEPROM(4);	// load stored data
	}
	else
	{
		if(input(PIN_A1) == 0)	// mode 1, amp mode
		{
			printf("kousei/puro'guramu/kidousima'sita");
			putc(0x0D);
			delay_ms(3000);
			while(input(PIN_C4)==0) {}	//wait during busy
			if(input(PIN_A2) == 0)
			{
				printf("pureibo'tankara/yubi'o/hana'sitekudasai");
			 	putc(0x0D);
				delay_ms(4000);
			}
			while(input(PIN_C4)==0) {}	//wait during busy
			while(input(PIN_A2) == 0) {}	// wait play switch
			//
			while(input(PIN_C4)==0) {}	//busy
			printf("den'ryuusokuteimo'-dodesu");
			putc(0x0D);
			delay_ms(4000);
			while(input(PIN_C4)==0) {}	//busy
			printf("tya'imuga/naru'maeni/ pure'ibotan/o/osu'to");
			printf(" /syukka'jino/de-tani/modo'sima'su");
			putc(0x0D);
			delay_ms(2000);
			while(input(PIN_C4)==0) {}	//busy
			//
			for(j=0; j<3000; j=j+1)
			{
				if(input(PIN_A2) == 0)	// wait play switch
				{
					while(input(PIN_C4)==0) {}	//busy
					printf("#K");
					putc(0x0D);
					delay_ms(1000);
					printf("syukka'jino/ de'-tao/ sette'isima'sita");
					putc(0x0D);
					delay_ms(2000);
					while(input(PIN_C4)==0) {}	//busy
					//volt10_coef = 1.07;
					amp_coef = 1.00;
					WRITE_FLOAT_EEPROM(4, amp_coef);
					factory_a_data = 1;
					break;
				}	
				delay_ms(1);
			}	
			//delay_ms(3000);
			printf("#J");
			putc(0x0D);
			delay_ms(1000);
			if(factory_a_data == 0)
			{
				printf("hyakumiri/anpe'ao /setu'zokusi / pu're'ibo'tan;o /osite;/kuda'sai");
				putc(0x0D);
				while(input(PIN_A2) == 1) {}	// wait
				if(input(PIN_A2) == 0)	// wait play switch
				{
					readadc();
					amp_coef = 1280.59/value;	// should be around 1.1
					WRITE_FLOAT_EEPROM(4, amp_coef);
					printf("#K");
					putc(0x0D);
					delay_ms(1000);
					while(input(PIN_C4)==0) {}	//wait during busy
					printf("kouse'isima'sita");
					putc(0x0D);
					delay_ms(2000);
					while(input(PIN_C4)==0) {}	//wait during busy
				}
			}
			
		}
		else if(input(PIN_A0) == 0)	// mode 3, x10 volt mode
		{
			printf("kousei/puro'guramu/kidousima'sita");
			putc(0x0D);
			delay_ms(3000);
			while(input(PIN_C4)==0) {}	//wait during busy
			if(input(PIN_A2) == 0)
			{
				printf("pureibo'tankara/yubi'o/hana'sitekudasai");
			 	putc(0x0D);
				delay_ms(4000);
			}	
			while(input(PIN_C4)==0) {}	//wait during busy
			//
			while(input(PIN_A2) == 0) {}	// wait play switch
			//
			printf("den'a'tusokuteimo'-do/ nijyu'-bo'rutore'njidesu");
			putc(0x0D);
			delay_ms(4000);
			while(input(PIN_C4)==0) {}	//wait during busy
			printf("tya'imuga/naru'maeni/ pure'ibotan/o/osu'to");
			printf(" /syukka'jino/de-tani/modo'sima'su");
			putc(0x0D);
			delay_ms(2000);
			while(input(PIN_C4)==0) {}	//wait during busy
			//
			for(j=0; j<3000; j=j+1)
			{
				if(input(PIN_A2) == 0)	// wait play switch
				{
					printf("#K");
					putc(0x0D);
					delay_ms(1000);
					while(input(PIN_C4)==0) {}	//wait during busy
					printf("syukka'jino/ de'-tao/ sette'isima'sita");
					putc(0x0D);
					delay_ms(2000);
					while(input(PIN_C4)==0) {}	//wait during busy
					volt10_coef = 1.07;
					factory_v_data = 1;
					WRITE_FLOAT_EEPROM(0, volt10_coef);
					break;
				}	
				delay_ms(1);
			}	
			//delay_ms(3000);
			printf("#J");
			putc(0x0D);
			delay_ms(1000);
			if(factory_v_data == 0)
			{
				while(input(PIN_C4)==0) {}	//wait during busy
				printf("gobo'rutoo /setu'zokusi/ pu're'ibo'tan;o /osite;/kuda'sai");
				putc(0x0D);
				while(input(PIN_A2) == 1) {}	// wait
				if(input(PIN_A2) == 0)	// waint play switch
				{
					readadc();
					volt10_coef = 8003.66/value;	// should be around 1.1
					WRITE_FLOAT_EEPROM(0, volt10_coef);
					printf("#K");
					putc(0x0D);
					delay_ms(1000);
					while(input(PIN_C4)==0) {}	//wait during busy
					printf("kouse'isima'sita");
					putc(0x0D);
					delay_ms(2000);
					while(input(PIN_C4)==0) {}	//wait during busy
				}
			}	
		}	
		else	// 2 volt range, it doesn't need any calibration
		{
			printf("den'atusokuteimo'-do/ nibo'rutore'njidesu");
			putc(0x0D);
			delay_ms(3000);
			while(input(PIN_C4)==0) {}	//wait during busy
			printf("konore'njidewa/ kou'seiwa dekimase'n");
			putc(0x0D);
			delay_ms(5000);
			while(input(PIN_C4)==0) {}	//wait during busy
			if(input(PIN_A2) == 0)
			{
				printf("pureibo'tankara/yubi'o/hana'sitekudasai");
			 	putc(0x0D);
				delay_ms(2000);
				while(input(PIN_A2) == 0) {}	// stay here
			}
		}	
	}
}		
//
#separate
void blink_led()
{
    output_high(LED);
    delay_ms(200);
    output_low(LED);
    delay_ms(200);
    output_high(LED);
    delay_ms(200);
    output_low(LED);
    delay_ms(200);
    output_high(LED);
    delay_ms(200);
    output_low(LED);
    delay_ms(200);
}

